//--------------------------------------------------------------------------------------
// File: MeshFromMA.fx
//
// The effect file for the MeshFromMA sample.  
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------


#include "skin.vsh"


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
float3 g_vMaterialAmbient : Ambient = float3( 0.2f, 0.2f, 0.2f );   // Material's ambient color
float3 g_vMaterialDiffuse : Diffuse = float3( 0.8f, 0.8f, 0.8f );   // Material's diffuse color
float3 g_vMaterialSpecular : Specular = float3( 1.0f, 1.0f, 1.0f );  // Material's specular color
float  g_fMaterialAlpha : Opacity = 1.0f;
int    g_nMaterialShininess : SpecularPower = 32;

float4 lhtDir       = { 0.7f, 0.0f, 0.7f, 1.0f };  // Light Direction
float3 g_vLightColor : LightColor = float3( 1.0f, 1.0f, 1.0f );        // Light color
//float3 g_vLightPosition : LightPosition = float3( 50000.0f, 10000.0f, 50000.0f );   // Light position
float3 g_vCameraPosition : CameraPosition;

texture  g_MeshTexture : Texture;   // Color texture for mesh
            
float    g_fTime : Time;            // App's time in seconds
float4x4 g_mWorld : World;          // World matrix
float4x4 g_mWorldViewProjection : WorldViewProjection; // World * View * Projection matrix
float4x4 g_mWorldView : WorldView; // World * View * Projection matrix
float4x4 g_mViewProj : VIEWPROJECTION;



//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------
sampler MeshTextureSampler = 
sampler_state
{
    Texture = <g_MeshTexture>;    
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


//--------------------------------------------------------------------------------------
// Name: Projection
// Type: Vertex Shader Fragment
// Desc: Projection transform
//--------------------------------------------------------------------------------------
void Projection( float4 vPosObject: POSITION,
                 float3 vNormalObject: NORMAL,
                 float2 vTexCoordIn: TEXCOORD0,
                 out float4 vPosProj: POSITION,
                 out float2 vTexCoordOut: TEXCOORD0,
                 out float4 vColorOut: COLOR0,
                 uniform bool bSpecular
                )
{
    // Transform the position into world space for lighting, and projected space
    // for display
    float4 vPosWorld = mul( vPosObject, g_mWorld );
    vPosProj = mul( vPosObject, g_mWorldViewProjection );
    
    // Transform the normal into world space for lighting
    float3 vNormalWorld = mul( vNormalObject, (float3x3)g_mWorld );
    // ʗpɗʂ`悳Ƃ͖@𔽓]
    float3 vNormalWorldView = mul( vNormalObject, (float3x3)g_mWorldView );
	if( vNormalWorldView.z > 0 )
		vNormalWorld = -vNormalWorld;
    // Pass the texture coordinate
    vTexCoordOut = vTexCoordIn;
    
    // Compute the light vector
    //float3 vLight = normalize( g_vLightPosition - vPosWorld.xyz );
    
    // Compute the ambient and diffuse components of illumination
    vColorOut.rgb = g_vLightColor * g_vMaterialAmbient;
    vColorOut.rgb += g_vLightColor * g_vMaterialDiffuse * saturate( dot( lhtDir.xyz, vNormalWorld ) );
    
    // If enabled, calculate the specular term
    if( bSpecular )
    {
        float3 vCamera = normalize(vPosWorld.xyz - g_vCameraPosition);
        float3 vReflection = reflect( lhtDir.xyz, vNormalWorld );
        float  fPhongValue = saturate( dot( vReflection, vCamera ) );

        vColorOut.rgb += g_vMaterialSpecular * pow(fPhongValue, g_nMaterialShininess);
    }
        
    vColorOut.a = g_fMaterialAlpha;
}

//--------------------------------------------------------------------------------------
struct VS_INPUT
{
    float4  Pos             : POSITION;
    float3  BlendWeights    : BLENDWEIGHT;
    float4  BlendIndices    : BLENDINDICES;
    float3  Normal          : NORMAL;
    float3  Tex0            : TEXCOORD0;
};

struct VS_OUTPUT
{
    float4  Pos     : POSITION;
    float4  Diffuse : COLOR0;
    float2  Tex0    : TEXCOORD0;
};

//--------------------------------------------------------------------------------------

VS_OUTPUT VertSkinning( VS_INPUT i )
{
    VS_OUTPUT   o;
    float3      Pos = 0.0f;
    float3      Normal = 0.0f;
    float       LastWeight = 0.0f;
    
    // skin VB inputs
    VS_SKIN_INPUT vsi = { i.Pos, i.BlendWeights, i.BlendIndices, i.Normal };
    VS_SKIN_OUTPUT vso = VS_Skin( vsi, 3 );

    // transform position from world space into view and then projection space
    o.Pos = mul( float4( vso.vPos.xyz, 1.0f ), g_mViewProj );

    // normalize normals
    Normal = normalize( vso.vNor );

    // Shade (Ambient + etc.)
    o.Diffuse = float4( g_vMaterialAmbient.xyz + saturate( dot( Normal, lhtDir.xyz ) ) * g_vMaterialDiffuse.xyz, 1.0 );

    // copy the input texture coordinate through
    o.Tex0  = i.Tex0.xy;

    return o;
}



//--------------------------------------------------------------------------------------
// Name: Lighting
// Type: Pixel Shader
// Desc: Compute lighting and modulate the texture
//--------------------------------------------------------------------------------------
void Lighting( float2 vTexCoord: TEXCOORD0,
               float4 vColorIn: COLOR0,
               out float4 vColorOut: COLOR0,
               uniform bool bTexture )
{  
    vColorOut = vColorIn;
    
    // Sample and modulate the texture
    if( bTexture )
        vColorOut.rgb *= tex2D( MeshTextureSampler, vTexCoord );
}

//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique SmoothSkin
{
    pass P0
    {
        VertexShader = compile vs_1_1 VertSkinning();    
        PixelShader = compile ps_1_4 Lighting(false);    
    }
}

technique Specular
{
    pass P0
    {
        VertexShader = compile vs_1_1 Projection(true);    
        PixelShader = compile ps_1_4 Lighting(false);    
    }
}

technique NoSpecular
{
    pass P0
    {
        VertexShader = compile vs_1_1 Projection(false);    
        PixelShader = compile ps_1_4 Lighting(false);    
    }
}

technique TexturedSpecular
{
    pass P0
    {
        VertexShader = compile vs_1_1 Projection(true);    
        PixelShader = compile ps_1_4 Lighting(true);    
    }
}

technique TexturedNoSpecular
{
    pass P0
    {
        VertexShader = compile vs_1_1 Projection(false);    
        PixelShader = compile ps_1_4 Lighting(true);    
    }
}

